@@ -204,7 +204,7 @@ static gboolean
 hal_gpower_filter_error (GError **error)
 {
 	/* short cut for speed, no error */
-	if (*error == NULL) {
+	if (error == NULL || *error == NULL) {
 		return FALSE;
 	}
 
@@ -230,10 +230,9 @@ hal_gpower_filter_error (GError **error)
  * Uses org.freedesktop.Hal.Device.SystemPowerManagement.Suspend ()
  **/
 gboolean
-hal_gpower_suspend (HalGPower *power, guint wakeup)
+hal_gpower_suspend (HalGPower *power, guint wakeup, GError **error)
 {
 	guint retval = 0;
-	GError *error = NULL;
 	gboolean ret;
 	DBusGProxy *proxy;
 
@@ -245,25 +244,19 @@ hal_gpower_suspend (HalGPower *power, guint wakeup)
 		return FALSE;
 	}
 
-	ret = dbus_g_proxy_call (proxy, "Suspend", &error,
+	ret = dbus_g_proxy_call (proxy, "Suspend", error,
 				 G_TYPE_INT, wakeup,
 				 G_TYPE_INVALID,
 				 G_TYPE_UINT, &retval,
 				 G_TYPE_INVALID);
 	/* we might have to ignore the error */
-	if (hal_gpower_filter_error (&error)) {
+	if (error != NULL && hal_gpower_filter_error (error) == TRUE) {
 		return TRUE;
 	}
-	if (error) {
-		g_warning ("ERROR: %s", error->message);
-		g_error_free (error);
+	if (retval != 0) {
+		g_warning ("Suspend failed in a horrible way!");
 	}
-	if (ret == FALSE || retval != 0) {
-		/* abort as the DBUS method failed */
-		g_warning ("Suspend failed!");
-		return FALSE;
-	}
-	return TRUE;
+	return ret;
 }
 
 /**
@@ -277,10 +270,9 @@ hal_gpower_suspend (HalGPower *power, guint wakeup)
  * with no arguments.
  **/
 static gboolean
-hal_gpower_pm_method_void (HalGPower *power, const gchar *method)
+hal_gpower_pm_method_void (HalGPower *power, const gchar *method, GError **error)
 {
 	guint retval = 0;
-	GError *error = NULL;
 	gboolean ret;
 	DBusGProxy *proxy;
 
@@ -297,24 +289,18 @@ hal_gpower_pm_method_void (HalGPower *power, const gchar *method)
 		return FALSE;
 	}
 
-	ret = dbus_g_proxy_call (proxy, method, &error,
+	ret = dbus_g_proxy_call (proxy, method, error,
 				 G_TYPE_INVALID,
 				 G_TYPE_UINT, &retval,
 				 G_TYPE_INVALID);
 	/* we might have to ignore the error */
-	if (hal_gpower_filter_error (&error)) {
+	if (error != NULL && hal_gpower_filter_error (error) == TRUE) {
 		return TRUE;
 	}
-	if (error) {
-		g_warning ("ERROR: %s", error->message);
-		g_error_free (error);
+	if (retval != 0) {
+		g_warning ("%s failed in a horrible way!", method);
 	}
-	if (ret == FALSE || retval != 0) {
-		/* abort as the DBUS method failed */
-		g_warning ("%s failed!", method);
-		return FALSE;
-	}
-	return TRUE;
+	return ret;
 }
 
 /**
@@ -326,10 +312,10 @@ hal_gpower_pm_method_void (HalGPower *power, const gchar *method)
  * Uses org.freedesktop.Hal.Device.SystemPowerManagement.Hibernate ()
  **/
 gboolean
-hal_gpower_hibernate (HalGPower *power)
+hal_gpower_hibernate (HalGPower *power, GError **error)
 {
 	g_return_val_if_fail (LIBHAL_IS_GPOWER (power), FALSE);
-	return hal_gpower_pm_method_void (power, "Hibernate");
+	return hal_gpower_pm_method_void (power, "Hibernate", error);
 }
 
 /**
@@ -340,10 +326,10 @@ hal_gpower_hibernate (HalGPower *power)
  * Uses org.freedesktop.Hal.Device.SystemPowerManagement.Shutdown ()
  **/
 gboolean
-hal_gpower_shutdown (HalGPower *power)
+hal_gpower_shutdown (HalGPower *power, GError **error)
 {
 	g_return_val_if_fail (LIBHAL_IS_GPOWER (power), FALSE);
-	return hal_gpower_pm_method_void (power, "Shutdown");
+	return hal_gpower_pm_method_void (power, "Shutdown", error);
 }
 
 /**
@@ -355,10 +341,10 @@ hal_gpower_shutdown (HalGPower *power)
  * Uses org.freedesktop.Hal.Device.SystemPowerManagement.Reboot ()
  **/
 gboolean
-hal_gpower_reboot (HalGPower *power)
+hal_gpower_reboot (HalGPower *power, GError **error)
 {
 	g_return_val_if_fail (LIBHAL_IS_GPOWER (power), FALSE);
-	return hal_gpower_pm_method_void (power, "Reboot");
+	return hal_gpower_pm_method_void (power, "Reboot", error);
 }
 
 /**
@@ -398,16 +384,10 @@ hal_gpower_enable_power_save (HalGPower *power, gboolean enable)
 				 G_TYPE_INVALID,
 				 G_TYPE_UINT, &retval,
 				 G_TYPE_INVALID);
-	if (error) {
-		g_warning ("ERROR: %s", error->message);
-		g_error_free (error);
+	if (retval != 0) {
+		g_warning ("SetPowerSave failed in a horrible way!");
 	}
-	if (ret == FALSE || retval != 0) {
-		/* abort as the DBUS method failed */
-		g_warning ("SetPowerSave failed!");
-		return FALSE;
-	}
-	return TRUE;
+	return ret;
 }
 
 /**
