@@ -26,6 +26,9 @@
 
 G_BEGIN_DECLS
 
+/* change general/installed_schema whenever adding or moving keys */
+#define GPM_CONF_SCHEMA_ID	3
+
 #define GPM_TYPE_CONF		(gpm_conf_get_type ())
 #define GPM_CONF(o)		(G_TYPE_CHECK_INSTANCE_CAST ((o), GPM_TYPE_CONF, GpmConf))
 #define GPM_CONF_CLASS(k)	(G_TYPE_CHECK_CLASS_CAST((k), GPM_TYPE_CONF, GpmConfClass))
@@ -78,6 +81,7 @@ G_BEGIN_DECLS
 
 /* general */
 #define GPM_CONF_DEBUG				GPM_CONF_DIR "/general/debug"
+#define GPM_CONF_SCHEMA_VERSION			GPM_CONF_DIR "/general/installed_schema"
 #define GPM_CONF_CAN_SUSPEND			GPM_CONF_DIR "/general/can_suspend"
 #define GPM_CONF_CAN_HIBERNATE			GPM_CONF_DIR "/general/can_hibernate"
 #define GPM_CONF_USE_TIME_POLICY		GPM_CONF_DIR "/general/use_time_for_policy"
@@ -98,7 +102,8 @@ G_BEGIN_DECLS
 #define GPM_CONF_LOCK_ON_BLANK_SCREEN		GPM_CONF_DIR "/lock/blank_screen"
 #define GPM_CONF_LOCK_ON_SUSPEND		GPM_CONF_DIR "/lock/suspend"
 #define GPM_CONF_LOCK_ON_HIBERNATE		GPM_CONF_DIR "/lock/hibernate"
-#define GPM_CONF_LOCK_GNOME_KEYRING		GPM_CONF_DIR "/lock/gnome_keyring"
+#define GPM_CONF_LOCK_GNOME_KEYRING_SUSPEND	GPM_CONF_DIR "/lock/gnome_keyring_suspend"
+#define GPM_CONF_LOCK_GNOME_KEYRING_HIBERNATE	GPM_CONF_DIR "/lock/gnome_keyring_hibernate"
 
 /* low power */
 #define GPM_CONF_LOWPOWER_AC			GPM_CONF_DIR "/lowpower/on_ac"
@@ -113,6 +118,7 @@ G_BEGIN_DECLS
 #define GPM_CONF_NOTIFY_SLEEP_FAILED		GPM_CONF_DIR "/notify/sleep_failed"
 #define GPM_CONF_NOTIFY_LOW_POWER		GPM_CONF_DIR "/notify/low_power"
 #define GPM_CONF_NOTIFY_ESTIMATED_DATA		GPM_CONF_DIR "/notify/estimated_data"
+#define GPM_CONF_NOTIFY_INHIBIT_LID		GPM_CONF_DIR "/notify/inhibit_lid"
 
 /* statistics */
 #define GPM_CONF_STATS_SHOW_AXIS_LABELS		GPM_CONF_DIR "/statistics/show_axis_labels"
@@ -147,6 +153,11 @@ G_BEGIN_DECLS
 /* okay to change at runtime */
 #define GPM_CONF_GNOME_SS_PM_DELAY		"/apps/gnome-screensaver/power_management_delay"
 
+/* gnome-screensaver */
+#define GS_CONF_DIR				"/apps/gnome-screensaver"
+#define GS_PREF_LOCK_ENABLED			GS_CONF_DIR "/lock_enabled"
+#define GS_PREF_IDLE_DELAY			GS_CONF_DIR "/idle_delay"
+
 typedef struct GpmConfPrivate GpmConfPrivate;
 
 typedef struct
