@@ -41,7 +41,7 @@
 #include <libgnomeui/gnome-client.h>
 
 #include <libhal-gpower.h>
-#include <libgpm.h>
+#include <libhal-gmanager.h>
 
 #include "gpm-ac-adapter.h"
 #include "gpm-button.h"
@@ -62,6 +62,7 @@
 #include "gpm-srv-brightness-kbd.h"
 #include "gpm-srv-screensaver.h"
 #include "gpm-stock-icons.h"
+#include "gpm-prefs-server.h"
 #include "gpm-sound.h"
 #include "gpm-tray-icon.h"
 #include "gpm-engine.h"
@@ -85,6 +86,7 @@ struct GpmManagerPrivate
 	GpmDpms			*dpms;
 	GpmIdle			*idle;
 	GpmInfo			*info;
+	GpmPrefsServer		*prefs_server;
 	GpmInhibit		*inhibit;
 	GpmNotify		*notify;
 	GpmControl		*control;
@@ -215,13 +217,13 @@ gpm_manager_sync_policy_sleep (GpmManager *manager)
 	on_ac = gpm_ac_adapter_is_present (manager->priv->ac_adapter);
 
 	if (on_ac == TRUE) {
-		gpm_conf_get_uint (manager->priv->conf, GPM_CONF_AC_SLEEP_COMPUTER, &sleep_computer);
-		gpm_conf_get_uint (manager->priv->conf, GPM_CONF_AC_SLEEP_DISPLAY, &sleep_display);
-		gpm_conf_get_bool (manager->priv->conf, GPM_CONF_AC_LOWPOWER, &power_save);
+		gpm_conf_get_uint (manager->priv->conf, GPM_CONF_TIMEOUT_SLEEP_COMPUTER_AC, &sleep_computer);
+		gpm_conf_get_uint (manager->priv->conf, GPM_CONF_TIMEOUT_SLEEP_DISPLAY_AC, &sleep_display);
+		gpm_conf_get_bool (manager->priv->conf, GPM_CONF_LOWPOWER_AC, &power_save);
 	} else {
-		gpm_conf_get_uint (manager->priv->conf, GPM_CONF_BATT_SLEEP_COMPUTER, &sleep_computer);
-		gpm_conf_get_uint (manager->priv->conf, GPM_CONF_BATT_SLEEP_DISPLAY, &sleep_display);
-		gpm_conf_get_bool (manager->priv->conf, GPM_CONF_BATT_LOWPOWER, &power_save);
+		gpm_conf_get_uint (manager->priv->conf, GPM_CONF_TIMEOUT_SLEEP_COMPUTER_BATT, &sleep_computer);
+		gpm_conf_get_uint (manager->priv->conf, GPM_CONF_TIMEOUT_SLEEP_DISPLAY_BATT, &sleep_display);
+		gpm_conf_get_bool (manager->priv->conf, GPM_CONF_LOWPOWER_BATT, &power_save);
 	}
 
 	hal_gpower_enable_power_save (manager->priv->hal_power, power_save);
@@ -412,15 +414,8 @@ manager_policy_do (GpmManager  *manager,
 {
 	gchar *action = NULL;
 
-	gpm_debug ("policy: %s", policy);
-	gpm_conf_get_string (manager->priv->conf, policy, &action);
-
-	if (action == NULL) {
-		return FALSE;
-	}
-
+	/* error msg timeout not valid */
 	if (gpm_control_is_policy_timout_valid (manager->priv->control) == FALSE) {
-		/* error msg timeout not valid */
 		gpm_notify_display (manager->priv->notify,
 				    _("Action forbidden"),
 				    _("Policy timeout is not valid. Please wait a few seconds and try again."),
@@ -430,15 +425,22 @@ manager_policy_do (GpmManager  *manager,
 		return FALSE;
 	}
 
+	gpm_debug ("policy: %s", policy);
+	gpm_conf_get_string (manager->priv->conf, policy, &action);
+
+	if (action == NULL) {
+		return FALSE;
+	}
+
 	if (strcmp (action, ACTION_NOTHING) == 0) {
 		gpm_info_explain_reason (manager->priv->info, GPM_EVENT_NOTIFICATION,
 					_("Doing nothing"), reason);
 
 	} else if (strcmp (action, ACTION_SUSPEND) == 0) {
-		gpm_manager_action_suspend (manager, "unknown");
+		gpm_manager_action_suspend (manager, reason);
 
 	} else if (strcmp (action, ACTION_HIBERNATE) == 0) {
-		gpm_manager_action_hibernate (manager, "unknown");
+		gpm_manager_action_hibernate (manager, reason);
 
 	} else if (strcmp (action, ACTION_BLANK) == 0) {
 		gpm_manager_blank_screen (manager, NULL);
@@ -612,6 +614,19 @@ gpm_manager_can_shutdown (GpmManager *manager,
 	return gpm_control_allowed_shutdown (manager->priv->control, can_shutdown, error);
 }
 
+/**
+ * gpm_manager_get_preferences_options:
+ **/
+gboolean
+gpm_manager_get_preferences_options (GpmManager *manager,
+				     gint       *capability,
+				     GError    **error)
+{
+	g_return_val_if_fail (manager != NULL, FALSE);
+	g_return_val_if_fail (GPM_IS_MANAGER (manager), FALSE);
+	return gpm_prefs_server_get_capability (manager->priv->prefs_server, capability);
+}
+
 /**
  * gpm_manager_get_power_save_status:
  *
@@ -692,9 +707,9 @@ idle_do_sleep (GpmManager *manager)
 	on_ac = gpm_ac_adapter_is_present (manager->priv->ac_adapter);
 
 	if (on_ac == TRUE) {
-		gpm_conf_get_string (manager->priv->conf, GPM_CONF_AC_SLEEP_TYPE, &action);
+		gpm_conf_get_string (manager->priv->conf, GPM_CONF_ACTIONS_SLEEP_TYPE_AC, &action);
 	} else {
-		gpm_conf_get_string (manager->priv->conf, GPM_CONF_BATT_SLEEP_TYPE, &action);
+		gpm_conf_get_string (manager->priv->conf, GPM_CONF_ACTIONS_SLEEP_TYPE_BATT, &action);
 	}
 
 	if (action == NULL) {
@@ -748,24 +763,12 @@ idle_changed_cb (GpmIdle    *idle,
 		 GpmIdleMode mode,
 		 GpmManager *manager)
 {
-	gboolean laptop_using_ext_mon;
-
-	/*
-	 * If external monitor connected we shouldn't ignore idle when lid closed.
-	 * Until HAL is able to detect which monitors are connected, control
-	 * behavior through gconf-key.
-	 * Details here: http://bugzilla.gnome.org/show_bug.cgi?id=365016
-	 */
-	gpm_conf_get_bool (manager->priv->conf, GPM_CONF_LAPTOP_USES_EXT_MON, &laptop_using_ext_mon);
-
-	/*
-	 * Ignore timeout events when the lid is closed, as the DPMS is
+	/* Ignore timeout events when the lid is closed, as the DPMS is
 	 * already off, and we don't want to perform policy actions or re-enable
 	 * the screen when the user moves the mouse on systems that do not
 	 * support hardware blanking.
-	 * Details are here: https://launchpad.net/malone/bugs/22522
-	 */
-	if (gpm_button_is_lid_closed (manager->priv->button) == TRUE && laptop_using_ext_mon == FALSE) {
+	 * Details are here: https://launchpad.net/malone/bugs/22522 */
+	if (gpm_button_is_lid_closed (manager->priv->button) == TRUE) {
 		gpm_debug ("lid is closed, so we are ignoring idle state changes");
 		return;
 	}
@@ -862,23 +865,52 @@ lid_button_pressed (GpmManager *manager,
 		    gboolean    pressed)
 {
 	gboolean on_ac;
+	gboolean has_inhibit;
+	gboolean do_policy;
+	gchar *action;
 
 	on_ac = gpm_ac_adapter_is_present (manager->priv->ac_adapter);
 
-	if (pressed == TRUE) {
-		if (on_ac == TRUE) {
-			gpm_debug ("Performing AC policy");
-			manager_policy_do (manager, GPM_CONF_AC_BUTTON_LID,
-					   _("the lid has been closed on ac power"));
-		} else {
-			gpm_debug ("Performing battery policy");
-			manager_policy_do (manager, GPM_CONF_BATT_BUTTON_LID,
-					   _("the lid has been closed on battery power"));
-		}
-	} else {
+	if (pressed == FALSE) {
 		/* we turn the lid dpms back on unconditionally */
 		gpm_manager_unblank_screen (manager, NULL);
+		return;
 	}
+
+	if (on_ac == TRUE) {
+		gpm_debug ("Performing AC policy");
+		manager_policy_do (manager, GPM_CONF_BUTTON_LID_AC,
+				   _("the lid has been closed on ac power"));
+		return;
+	}
+
+	/* default */
+	do_policy = TRUE;
+
+	/* are we inhibited? */
+	gpm_inhibit_has_inhibit (manager->priv->inhibit, &has_inhibit, NULL);
+
+	/* do not do lid close action if suspend (or hibernate) */
+	if (has_inhibit == TRUE) {
+		/* get the policy action for battery */
+		gpm_conf_get_string (manager->priv->conf, GPM_CONF_BUTTON_LID_BATT, &action);
+
+		/* if we are trying to suspend or hibernate then don't do action */
+		if ((strcmp (action, ACTION_SUSPEND) == 0) ||
+		    (strcmp (action, ACTION_HIBERNATE) == 0)) {
+			do_policy = FALSE;
+		}
+		g_free (action);
+	}
+
+	if (do_policy == FALSE) {
+		gpm_debug ("Not doing lid policy action as inhibited as set to sleep");
+		return;
+	}
+
+	gpm_debug ("Performing battery policy");
+	manager_policy_do (manager, GPM_CONF_BUTTON_LID_BATT,
+			   _("the lid has been closed on battery power"));
 }
 
 /**
@@ -947,9 +979,9 @@ ac_adapter_changed_cb (GpmAcAdapter *ac_adapter,
 
 	on_ac = gpm_ac_adapter_is_present (manager->priv->ac_adapter);
 	if (on_ac == TRUE) {
-		gpm_conf_get_bool (manager->priv->conf, GPM_CONF_AC_LOWPOWER, &power_save);
+		gpm_conf_get_bool (manager->priv->conf, GPM_CONF_LOWPOWER_AC, &power_save);
 	} else {
-		gpm_conf_get_bool (manager->priv->conf, GPM_CONF_BATT_LOWPOWER, &power_save);
+		gpm_conf_get_bool (manager->priv->conf, GPM_CONF_LOWPOWER_BATT, &power_save);
 	}
 	if (manager->priv->low_power != power_save) {
 		g_signal_emit (manager, signals [POWER_SAVE_STATUS_CHANGED], 0, power_save);
@@ -958,7 +990,7 @@ ac_adapter_changed_cb (GpmAcAdapter *ac_adapter,
 
 	/* We do the lid close on battery action if the ac_adapter is removed
 	   when the laptop is closed and on battery. Fixes #331655 */
-	gpm_conf_get_bool (manager->priv->conf, GPM_CONF_BATT_EVENT_WHEN_CLOSED, &event_when_closed);
+	gpm_conf_get_bool (manager->priv->conf, GPM_CONF_ACTIONS_SLEEP_WHEN_CLOSED, &event_when_closed);
 
 	/* We keep track of the lid state so we can do the
 	   lid close on battery action if the ac_adapter is removed when the laptop
@@ -966,7 +998,7 @@ ac_adapter_changed_cb (GpmAcAdapter *ac_adapter,
 	if (event_when_closed == TRUE &&
 	    on_ac == FALSE &&
 	    gpm_button_is_lid_closed (manager->priv->button)) {
-		manager_policy_do (manager, GPM_CONF_BATT_BUTTON_LID,
+		manager_policy_do (manager, GPM_CONF_BUTTON_LID_BATT,
 				   _("the lid has been closed, and the ac adapter "
 				     "removed (and gconf is okay)"));
 	}
@@ -989,7 +1021,7 @@ static gboolean
 manager_critical_action_do (GpmManager *manager)
 {
 	manager_policy_do (manager,
-			   GPM_CONF_BATT_CRITICAL,
+			   GPM_CONF_ACTIONS_CRITICAL_BATT,
 			   _("battery is critically low"));
 	return FALSE;
 }
@@ -1061,8 +1093,8 @@ conf_key_changed_cb (GpmConf     *conf,
 		     const gchar *key,
 		     GpmManager  *manager)
 {
-	if (strcmp (key, GPM_CONF_BATT_SLEEP_COMPUTER) == 0 ||
-		   strcmp (key, GPM_CONF_AC_SLEEP_COMPUTER) == 0) {
+	if (strcmp (key, GPM_CONF_TIMEOUT_SLEEP_COMPUTER_BATT) == 0 ||
+		   strcmp (key, GPM_CONF_TIMEOUT_SLEEP_COMPUTER_AC) == 0) {
 
 		gpm_manager_sync_policy_sleep (manager);
 	}
@@ -1323,6 +1355,9 @@ gpm_engine_charge_low_cb (GpmEngine      *engine,
 	} else if (kind == GPM_CELL_UNIT_KIND_PDA) {
 		title = _("PDA battery low");
 		message = g_strdup_printf (_("The PDA attached to this computer is low in power (%d%%)"), unit->percentage);
+	} else if (kind == GPM_CELL_UNIT_KIND_PHONE) {
+		title = _("Cell phone battery low");
+		message = g_strdup_printf (_("The cell phone attached to this computer is low in power (%d%%)"), unit->percentage);
 	}
 
 	/* get correct icon */
@@ -1377,7 +1412,11 @@ gpm_engine_charge_critical_cb (GpmEngine      *engine,
 		time_text = gpm_manager_get_time_until_action_text (manager);
 
 		/* we have to do different warnings depending on the policy */
-		gpm_conf_get_string (manager->priv->conf, GPM_CONF_BATT_CRITICAL, &action);
+		gpm_conf_get_string (manager->priv->conf, GPM_CONF_ACTIONS_CRITICAL_BATT, &action);
+		if (action == NULL) {
+			gpm_warning ("schema invalid!");
+			action = g_strdup (ACTION_NOTHING);
+		}
 
 		/* use different text for different actions */
 		if (strcmp (action, ACTION_NOTHING) == 0) {
@@ -1422,6 +1461,11 @@ gpm_engine_charge_critical_cb (GpmEngine      *engine,
 		message = g_strdup_printf (_("The PDA attached to this computer is very low in power (%d%%). "
 					     "This device will soon stop functioning if not charged."),
 					   unit->percentage);
+	} else if (kind == GPM_CELL_UNIT_KIND_PHONE) {
+		title = _("Cell phone battery low");
+		message = g_strdup_printf (_("Your cell phone is very low in power (%d%%). "
+					     "This device will soon stop functioning if not charged."),
+					   unit->percentage);
 	}
 
 	/* get correct icon */
@@ -1452,7 +1496,7 @@ gpm_engine_charge_action_cb (GpmEngine      *engine,
 		title = _("Laptop battery critically low");
 
 		/* we have to do different warnings depending on the policy */
-		gpm_conf_get_string (manager->priv->conf, GPM_CONF_BATT_CRITICAL, &action);
+		gpm_conf_get_string (manager->priv->conf, GPM_CONF_ACTIONS_CRITICAL_BATT, &action);
 
 		/* use different text for different actions */
 		if (strcmp (action, ACTION_NOTHING) == 0) {
@@ -1484,7 +1528,7 @@ gpm_engine_charge_action_cb (GpmEngine      *engine,
 		title = _("UPS critically low");
 
 		/* we have to do different warnings depending on the policy */
-		gpm_conf_get_string (manager->priv->conf, GPM_CONF_UPS_CRITICAL, &action);
+		gpm_conf_get_string (manager->priv->conf, GPM_CONF_ACTIONS_CRITICAL_UPS, &action);
 
 		/* use different text for different actions */
 		if (strcmp (action, ACTION_NOTHING) == 0) {
@@ -1519,6 +1563,57 @@ gpm_engine_charge_action_cb (GpmEngine      *engine,
 	g_free (message);
 }
 
+/**
+ * has_inhibit_changed_cb:
+ **/
+static void
+has_inhibit_changed_cb (GpmInhibit *inhibit,
+			gboolean    has_inhibit,
+		        GpmManager *manager)
+{
+	HalGManager *hal_manager;
+	gboolean is_laptop;
+	gboolean show_inhibit_lid;
+	gchar *action = NULL;
+
+	/* we don't care about uninhibits */
+	if (has_inhibit == FALSE) {
+		return;
+	}
+
+	/* only show this if specified in gconf */
+	gpm_conf_get_bool (manager->priv->conf, GPM_CONF_NOTIFY_INHIBIT_LID, &show_inhibit_lid);
+
+	/* we've already shown the UI and been clicked */
+	if (show_inhibit_lid == FALSE) {
+		return;
+	}
+
+	hal_manager = hal_gmanager_new ();
+	is_laptop = hal_gmanager_is_laptop (hal_manager);
+	g_object_unref (hal_manager);
+
+	/* we don't warn for desktops, as they do not have a lid... */
+	if (is_laptop == FALSE) {
+		return;
+	}
+
+	/* get the policy action for battery */
+	gpm_conf_get_string (manager->priv->conf, GPM_CONF_BUTTON_LID_BATT, &action);
+
+	if (action == NULL) {
+		return;
+	}
+
+	/* if the policy on lid close is sleep then show a warning */
+	if ((strcmp (action, ACTION_SUSPEND) == 0) ||
+	    (strcmp (action, ACTION_HIBERNATE) == 0)) {
+		gpm_notify_inhibit_lid (manager->priv->notify);
+	}
+
+	g_free (action);
+}
+
 /**
  * gpm_manager_init:
  * @manager: This class instance
@@ -1531,6 +1626,8 @@ gpm_manager_init (GpmManager *manager)
 	GpmEngineCollection *collection;
 	GError *error = NULL;
 	gboolean on_ac;
+	gboolean ret;
+	guint version;
 
 	manager->priv = GPM_MANAGER_GET_PRIVATE (manager);
 	connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
@@ -1538,10 +1635,27 @@ gpm_manager_init (GpmManager *manager)
 	/* do some actions even when killed */
 	g_atexit (gpm_manager_at_exit);
 
+	/* this is a singleton, so we keep a master copy open here */
+	manager->priv->prefs_server = gpm_prefs_server_new ();
+
+	manager->priv->notify = gpm_notify_new ();
 	manager->priv->conf = gpm_conf_new ();
 	g_signal_connect (manager->priv->conf, "value-changed",
 			  G_CALLBACK (conf_key_changed_cb), manager);
 
+	/* check to see if the user has installed the schema properly */
+	ret = gpm_conf_get_uint (manager->priv->conf, GPM_CONF_SCHEMA_VERSION, &version);
+	if (ret == FALSE || version != GPM_CONF_SCHEMA_ID) {
+		gpm_notify_display (manager->priv->notify,
+				    _("Install problem!"),
+				    _("The configuration defaults for GNOME Power Manager have not been installed correctly.\n"
+				      "Please contact your computer administrator."),
+				    GPM_NOTIFY_TIMEOUT_LONG,
+				    GTK_STOCK_DIALOG_WARNING,
+				    GPM_NOTIFY_URGENCY_NORMAL);
+		gpm_error ("no gconf schema installed!");
+	}
+
 	/* we use ac_adapter so we can poke the screensaver and throttle */
 	manager->priv->ac_adapter = gpm_ac_adapter_new ();
 	g_signal_connect (manager->priv->ac_adapter, "ac-adapter-changed",
@@ -1550,9 +1664,9 @@ gpm_manager_init (GpmManager *manager)
 	/* coldplug so we are in the correct state at startup */
 	on_ac = gpm_ac_adapter_is_present (manager->priv->ac_adapter);
 	if (on_ac == TRUE) {
-		gpm_conf_get_bool (manager->priv->conf, GPM_CONF_AC_LOWPOWER, &manager->priv->low_power);
+		gpm_conf_get_bool (manager->priv->conf, GPM_CONF_LOWPOWER_AC, &manager->priv->low_power);
 	} else {
-		gpm_conf_get_bool (manager->priv->conf, GPM_CONF_BATT_LOWPOWER, &manager->priv->low_power);
+		gpm_conf_get_bool (manager->priv->conf, GPM_CONF_LOWPOWER_BATT, &manager->priv->low_power);
 	}
 
 	manager->priv->button = gpm_button_new ();
@@ -1592,11 +1706,12 @@ gpm_manager_init (GpmManager *manager)
 	gpm_idle_set_check_cpu (manager->priv->idle, check_type_cpu);
 
 	manager->priv->dpms = gpm_dpms_new ();
-	manager->priv->notify = gpm_notify_new ();
 
 	/* use a class to handle the complex stuff */
 	gpm_debug ("creating new inhibit instance");
 	manager->priv->inhibit = gpm_inhibit_new ();
+	g_signal_connect (manager->priv->inhibit, "has-inhibit-changed",
+			  G_CALLBACK (has_inhibit_changed_cb), manager);
 	/* add the interface */
 	dbus_g_object_type_install_info (GPM_TYPE_INHIBIT, &dbus_glib_gpm_inhibit_object_info);
 	dbus_g_connection_register_g_object (connection, GPM_DBUS_PATH_INHIBIT,
@@ -1688,6 +1803,8 @@ gpm_manager_finalize (GObject *object)
 	g_object_unref (manager->priv->screensaver);
 	g_object_unref (manager->priv->notify);
 	g_object_unref (manager->priv->srv_screensaver);
+	g_object_unref (manager->priv->prefs_server);
+	g_object_unref (manager->priv->control);
 
 	/* optional gobjects */
 	if (manager->priv->button) {
