@@ -42,9 +42,9 @@
 #include "gpm-array.h"
 #include "gpm-array-float.h"
 #include "gpm-dpms.h"
-#include "gpm-common.h"
 #include "gpm-load.h"
 #include "gpm-debug.h"
+#include "gpm-control.h"
 
 #include "gpm-profile.h"
 
@@ -63,6 +63,7 @@ struct GpmProfilePrivate
 	GpmAcAdapter		*ac_adapter;
 	GpmLoad			*load;
 	GpmDpms			*dpms;
+	GpmControl		*control;
 	GTimer			*timer;
 	GpmArray		*present_array_accuracy;
 	GpmArray		*present_array_data;
@@ -75,6 +76,7 @@ struct GpmProfilePrivate
 	gboolean		 discharging;
 	gboolean		 lcd_on;
 	gboolean		 data_valid;
+	gboolean		 use_guessing;
 	guint			 last_percentage;
 	gchar			*config_id;
 };
@@ -130,6 +132,40 @@ gpm_profile_get_data_file (GpmProfile *profile, gboolean discharge)
 				g_get_home_dir (), profile->priv->config_id, suffix);
 }
 
+/**
+ * gpm_profile_print:
+ *
+ * @profile: This class
+ */
+void
+gpm_profile_print (GpmProfile *profile)
+{
+	guint i;
+	GArray *array_data;
+	GArray *array_accuracy;
+	gfloat value_data;
+	gfloat value_accuracy;
+
+	g_return_if_fail (profile != NULL);
+	g_return_if_fail (GPM_IS_PROFILE (profile));
+
+	/* get the correct data */
+	if (profile->priv->discharging == TRUE) {
+		array_data = profile->priv->float_data_discharge;
+		array_accuracy = profile->priv->float_accuracy_discharge;
+	} else {
+		array_data = profile->priv->float_data_charge;
+		array_accuracy = profile->priv->float_accuracy_charge;
+	}
+
+	/* copy the data field into the accuracy y field */
+	for (i=0; i<100; i++) {
+		value_data = gpm_array_float_get (array_data, i);
+		value_accuracy = gpm_array_float_get (array_accuracy, i);
+		g_print ("[%i], %f (%f)\n", i, value_data, value_accuracy);
+	}
+}
+
 /**
  * gpm_profile_get_data_time_percent:
  *
@@ -139,38 +175,34 @@ GpmArray *
 gpm_profile_get_data_time_percent (GpmProfile *profile, gboolean discharge)
 {
 	guint i;
-	GArray *array;
-	gfloat value;
-	guint32 colour_active;
-	guint32 colour_nonactive;
+	GArray *array_data;
+	GArray *array_accuracy;
+	gfloat value_data;
+	gfloat value_accuracy;
 
 	g_return_val_if_fail (profile != NULL, NULL);
 	g_return_val_if_fail (GPM_IS_PROFILE (profile), NULL);
 
-	if (discharge == TRUE) {
-		colour_active = GPM_COLOUR_RED;
-		colour_nonactive = GPM_COLOUR_DARK_RED;
-	} else {
-		colour_active = GPM_COLOUR_BLUE;
-		colour_nonactive = GPM_COLOUR_DARK_BLUE;
-	}
-
 	/* get the correct data */
 	if (discharge == TRUE) {
-		array = profile->priv->float_data_discharge;
+		array_data = profile->priv->float_data_discharge;
+		array_accuracy = profile->priv->float_accuracy_discharge;
 	} else {
-		array = profile->priv->float_data_charge;
+		array_data = profile->priv->float_data_charge;
+		array_accuracy = profile->priv->float_accuracy_charge;
 	}
 
 	/* copy the data field into the accuracy y field */
 	for (i=0; i<100; i++) {
-		value = gpm_array_float_get (array, i);
+		value_data = gpm_array_float_get (array_data, i);
+		value_accuracy = gpm_array_float_get (array_accuracy, i);
 		/* only set points that are not zero */
-		if (value > 0) {
-			gpm_array_set (profile->priv->present_array_data, i, i, value, colour_active);
-		} else {
-			/* set zero points a different colour */
-			gpm_array_set (profile->priv->present_array_data, i, i, value, colour_nonactive);
+		if (value_data == 0) {
+			gpm_array_set (profile->priv->present_array_data, i, i, value_data, GPM_COLOUR_BLUE);
+		} else if (value_accuracy == 0) {
+			gpm_array_set (profile->priv->present_array_data, i, i, value_data, GPM_COLOUR_DARK_GREY);
+		} else if (value_data > 0) {
+			gpm_array_set (profile->priv->present_array_data, i, i, value_data, GPM_COLOUR_RED);
 		}
 	}
 
@@ -289,12 +321,114 @@ gpm_profile_get_time (GpmProfile *profile, guint percentage, gboolean discharge)
 	return time;
 }
 
+/**
+ * gpm_profile_get_nonzero_accuracy_percent:
+ *
+ * @profile: This class
+ */
+static float
+gpm_profile_get_nonzero_accuracy_percent (GArray *array_data, GArray *array_accuracy)
+{
+	/* find the average "non-zero accuracy" data */
+	guint i;
+	guint length;
+	gfloat data;
+	gfloat accuracy;
+	guint length_average = 0;
+	gfloat average = 0;
+
+	length = array_data->len;
+	for (i=0; i<length; i++) {
+		data = g_array_index (array_data, gfloat, i);
+		accuracy = g_array_index (array_accuracy, gfloat, i);
+		if (accuracy > 0) {
+			average += data;
+			length_average++;
+		}
+	}
+	gpm_debug ("average=%f, length_average=%i", average, length_average);
+	if (average == 0 || length_average == 0) {
+		/* no data with accuracy */
+		gpm_warning ("no data");
+		return 0;
+	}
+	return average / (gfloat) length_average;
+}
+
+/**
+ * gpm_profile_set_zero_accuracy_average:
+ *
+ * @profile: This class
+ */
+static void
+gpm_profile_set_zero_accuracy_average (GArray *array_data,
+				       GArray *array_accuracy,
+				       gfloat  average,
+				       guint   start,
+				       guint   stop)
+{
+	/* set the zero accuracy points to the average value over a range */
+	guint i;
+	guint length;
+	gfloat accuracy;
+	guint width = 0;
+
+	length = array_data->len;
+	if (stop > length) {
+		stop = length;
+	}
+	for (i=start; i<stop; i++) {
+		accuracy = g_array_index (array_accuracy, gfloat, i);
+		if (accuracy == 0) {
+			/* we only do this if the width is not just one reading
+			   as some batteries map 64 states into percent */
+			if (width > 0) {
+				g_array_index (array_data, gfloat, i) = average;
+			}
+			width++;
+		} else {
+			width = 0;
+		}
+	}
+}
+
+/**
+ * gpm_profile_set_average_no_accuracy:
+ *
+ * @profile: This class
+ */
+static void
+gpm_profile_set_average_no_accuracy (GpmProfile *profile)
+{
+	GArray *array_data;
+	GArray *array_accuracy;
+	gfloat average;
+
+	/* get the correct data */
+	if (profile->priv->discharging == TRUE) {
+		array_data = profile->priv->float_data_discharge;
+		array_accuracy = profile->priv->float_accuracy_discharge;
+	} else {
+		array_data = profile->priv->float_data_charge;
+		array_accuracy = profile->priv->float_accuracy_charge;
+	}
+
+	/* find the data average of the non zero accuracy points */
+	average = gpm_profile_get_nonzero_accuracy_percent (array_data, array_accuracy);
+	gpm_debug ("estimated average = %f", average);
+
+	/* set the zero accuracy data to the average */
+	gpm_profile_set_zero_accuracy_average (array_data, array_accuracy, average, 5, 95);
+}
 
 /**
  * gpm_profile_save_percentage:
  *
  * @profile: This class
  * @percentage: new percentage value
+ * @time: seconds elapsed
+ * @measurement_accuracy: the percentage accuracy, where 100% is the
+ *			  best we can get...
  */
 static void
 gpm_profile_save_percentage (GpmProfile *profile,
@@ -321,8 +455,8 @@ gpm_profile_save_percentage (GpmProfile *profile,
 	data = gpm_array_float_get (array_data, percentage);
 	accuracy = gpm_array_float_get (array_accuracy, percentage);
 
-	/* if we have no data, then just use the new value */
-	if (data == 0) {
+	/* if we have no accuracy, then just use the new value */
+	if (accuracy == 0) {
 		data = time;
 	} else {
 		/* average the data so we converge to a common point */
@@ -339,6 +473,11 @@ gpm_profile_save_percentage (GpmProfile *profile,
 	}
 	gpm_array_float_set (array_accuracy, percentage, accuracy);
 
+	/* guess missing data between 5 and 95 */
+	if (profile->priv->use_guessing == TRUE) {
+		gpm_profile_set_average_no_accuracy (profile);
+	}
+
 	/* create a temp 2D array so we can save the CSV */
 	array = gpm_array_new ();
 	gpm_array_set_fixed_size (array, 100);
@@ -430,6 +569,16 @@ gpm_profile_register_percentage (GpmProfile *profile,
 	return TRUE;
 }
 
+/**
+ * gpm_profile_test_force_discharging:
+ */
+void
+gpm_profile_test_force_discharging (GpmProfile *profile,
+				    gboolean    is_discharging)
+{
+	profile->priv->discharging = is_discharging;
+}
+
 /**
  * gpm_profile_register_charging:
  */
@@ -521,8 +670,8 @@ dpms_mode_changed_cb (GpmDpms    *dpms,
 gboolean
 gpm_profile_delete_data (GpmProfile *profile, gboolean discharge)
 {
-	gchar *filename;
-	gint ret;
+	gchar *filename = NULL;
+	gboolean ret = FALSE;
 
 	g_return_val_if_fail (profile != NULL, FALSE);
 	g_return_val_if_fail (GPM_IS_PROFILE (profile), FALSE);
@@ -530,16 +679,26 @@ gpm_profile_delete_data (GpmProfile *profile, gboolean discharge)
 	/* check we have a profile loaded */
 	if (profile->priv->config_id == NULL) {
 		gpm_warning ("no config id set!");
-		return FALSE;
+		goto out;
 	}
 
+	/* create filename and check is exists */
 	filename = gpm_profile_get_data_file (profile, discharge);
-	ret = g_unlink (filename);
-	if (ret != 0) {
-		gpm_warning ("could not delete '%s'", filename);
-		return FALSE;
+	if (g_file_test (filename, G_FILE_TEST_EXISTS) == FALSE) {
+		/* seems insane, but we need this for the test suite */
+		ret = TRUE;
+		goto out;
 	}
-	return TRUE;
+
+	/* try to delete file */
+	if (g_unlink (filename) != 0) {
+		gpm_warning ("could not delete '%s'", filename);
+		goto out;
+	}
+	ret = TRUE;
+out:
+	g_free (filename);
+	return ret;
 }
 
 /**
@@ -584,7 +743,7 @@ gpm_profile_load_data (GpmProfile *profile, gboolean discharge)
 
 		gpm_debug ("no data found, generating initial (poor) data");
 		for (i=0;i<100;i++) {
-			/* assume average battery lasts 2 hours, but we are 0% accurate */
+			/* just set data to zero as we have _no_ idea */
 			gpm_array_set (array, i, i, 0, 0);
 		}
 
@@ -654,6 +813,17 @@ gpm_profile_set_config_id (GpmProfile  *profile,
 	return TRUE;
 }
 
+/**
+ * gpm_profile_use_guessing:
+ */
+gboolean
+gpm_profile_use_guessing (GpmProfile *profile,
+			  gboolean    use_guessing)
+{
+	profile->priv->use_guessing = use_guessing;
+	return TRUE;
+}
+
 /**
  * gpm_profile_get_accuracy:
  */
@@ -687,6 +857,18 @@ gpm_profile_get_accuracy (GpmProfile *profile,
 	return gpm_array_float_get (array, percentage);
 }
 
+/**
+ * control_sleep_action_cb:
+ **/
+static void
+control_sleep_action_cb (GpmControl      *control,
+			 GpmControlAction action,
+		         GpmProfile      *profile)
+{
+	/* the charge might have changed when suspended or hibernated */
+	profile->priv->data_valid = FALSE;
+}
+
 /**
  * gpm_profile_init:
  */
@@ -700,6 +882,7 @@ gpm_profile_init (GpmProfile *profile)
 	profile->priv->timer = g_timer_new ();
 	profile->priv->load = gpm_load_new ();
 	profile->priv->config_id = NULL;
+	profile->priv->use_guessing = TRUE;
 
 	profile->priv->ac_adapter = gpm_ac_adapter_new ();
 	g_signal_connect (profile->priv->ac_adapter, "ac-adapter-changed",
@@ -709,7 +892,6 @@ gpm_profile_init (GpmProfile *profile)
 	g_signal_connect (profile->priv->dpms, "mode-changed",
 			  G_CALLBACK (dpms_mode_changed_cb), profile);
 
-
 	/* used for data */
 	profile->priv->float_data_charge = gpm_array_float_new (100);
 	profile->priv->float_data_discharge = gpm_array_float_new (100);
@@ -722,6 +904,17 @@ gpm_profile_init (GpmProfile *profile)
 	gpm_array_set_fixed_size (profile->priv->present_array_data, 100);
 	gpm_array_set_fixed_size (profile->priv->present_array_accuracy, 100);
 
+	/* use the control object so we don't save new percentage points
+	 * after suspend or hibernate */
+	gpm_debug ("creating new control instance");
+	profile->priv->control = gpm_control_new ();
+	g_signal_connect (profile->priv->control, "sleep",
+			  G_CALLBACK (control_sleep_action_cb), profile);
+	g_signal_connect (profile->priv->control, "resume",
+			  G_CALLBACK (control_sleep_action_cb), profile);
+	g_signal_connect (profile->priv->control, "sleep-failure",
+			  G_CALLBACK (control_sleep_action_cb), profile);
+
 	/* default */
 	profile->priv->lcd_on = TRUE;
 	profile->priv->last_percentage = 100;
@@ -765,6 +958,7 @@ gpm_profile_finalize (GObject *object)
 	g_object_unref (profile->priv->load);
 	g_object_unref (profile->priv->dpms);
 	g_object_unref (profile->priv->ac_adapter);
+	g_object_unref (profile->priv->control);
 
 	gpm_array_float_free (profile->priv->float_data_charge);
 	gpm_array_float_free (profile->priv->float_data_discharge);
@@ -794,3 +988,239 @@ gpm_profile_new (void)
 	return GPM_PROFILE (gpm_profile_object);
 }
 
+/***************************************************************************
+ ***                          MAKE CHECK TESTS                           ***
+ ***************************************************************************/
+#ifdef GPM_BUILD_TESTS
+#include "gpm-self-test.h"
+
+static void
+reset_profile (GpmProfile *profile)
+{
+	gpm_profile_delete_data (profile, TRUE);
+	gpm_profile_delete_data (profile, FALSE);
+	gpm_profile_load_data (profile, TRUE);
+	gpm_profile_load_data (profile, FALSE);
+}
+
+void
+gpm_st_profile (GpmSelfTest *test)
+{
+	GpmProfile *profile;
+	gboolean ret;
+	gchar *filename;
+	gint i;
+	guint value;
+	gfloat fvalue;
+
+	if (gpm_st_start (test, "GpmProfile", CLASS_AUTO) == FALSE) {
+		return;
+	}
+
+	/************************************************************/
+	gpm_st_title (test, "make sure we get a non null profile");
+	profile = gpm_profile_new ();
+	if (profile != NULL) {
+		gpm_st_success (test, "got GpmProfile");
+	} else {
+		gpm_st_failed (test, "could not get GpmProfile");
+	}
+
+	/************************************************************/
+	gpm_st_title (test, "make sure we get a zero accuracy when no-id");
+	value = gpm_profile_get_accuracy (profile, 50);
+	if (value == 0) {
+		gpm_st_success (test, "got zero");
+	} else {
+		gpm_st_failed (test, "got %i", value);
+	}
+
+	/************************************************************/
+	gpm_st_title (test, "set config id");
+	ret = gpm_profile_set_config_id (profile, "test123");
+	if (ret == TRUE) {
+		gpm_st_success (test, "set type");
+	} else {
+		gpm_st_failed (test, "could not set type");
+	}
+
+
+	/************************************************************
+	 **              UTILITY FUNCTIONS                         **
+	 ************************************************************/
+
+	/************************************************************/
+	gpm_st_title (test, "get correct charging filename");
+	filename = gpm_profile_get_data_file (profile, FALSE);
+	if (strstr (filename, "/.gnome2/gnome-power-manager/profile-test123-charging.csv") != NULL) {
+		gpm_st_success (test, "got correct filename");
+	} else {
+		gpm_st_failed (test, "got incorrect filename: %s", filename);
+	}
+	g_free (filename);
+
+	/************************************************************/
+	gpm_st_title (test, "get correct discharging filename");
+	filename = gpm_profile_get_data_file (profile, TRUE);
+	if (strstr (filename, "/.gnome2/gnome-power-manager/profile-test123-discharging.csv") != NULL) {
+		gpm_st_success (test, "got correct filename");
+	} else {
+		gpm_st_failed (test, "got incorrect filename: %s", filename);
+	}
+	g_free (filename);
+
+	/* clear old profile */
+	reset_profile (profile);
+
+	/************************************************************/
+	gpm_st_title (test, "make sure we get a zero accuracy with a new dataset");
+	value = gpm_profile_get_accuracy (profile, 50);
+	if (value == 0) {
+		gpm_st_success (test, "got zero");
+	} else {
+		gpm_st_failed (test, "got %i (not zero!)", value);
+	}
+
+	/************************************************************/
+	gpm_st_title (test, "make sure we get a zero time with a new dataset");
+	value = gpm_profile_get_time (profile, 50, TRUE);
+	if (value == 0) {
+		gpm_st_success (test, "got zero");
+	} else {
+		gpm_st_failed (test, "got %i", value);
+	}
+
+	/************************************************************
+	 **                TYPICAL DISCHARGE                       **
+	 ************************************************************/
+
+	/************************************************************/
+	gpm_st_title (test, "force discharging");
+	gpm_profile_test_force_discharging (profile, TRUE);
+	gpm_st_success (test, NULL);
+
+	/************************************************************/
+	gpm_st_title (test, "ignore first point");
+	ret = gpm_profile_register_percentage (profile, 99);
+	if (ret == FALSE) {
+		gpm_st_success (test, "ignored first");
+	} else {
+		gpm_st_failed (test, "ignored second");
+	}
+
+	/************************************************************/
+	gpm_st_title (test, "make up discharging dataset 98-0 (perfect accuracy)");
+	for (i=98; i>=0; i--) {
+		gpm_profile_save_percentage (profile, i, 120, 100);
+	}
+	gpm_st_success (test, "put dataset");
+
+	/************************************************************/
+	gpm_st_title (test, "make sure we get a correct accuracy when a complete dataset");
+	value = gpm_profile_get_accuracy (profile, 50);
+	if (value == 20) {
+		gpm_st_success (test, "got correct average %i", value);
+	} else {
+		gpm_st_failed (test, "got incorrect average %i", value);
+	}
+
+	/************************************************************/
+	gpm_st_title (test, "make sure we get a correct discharge time when set");
+	value = gpm_profile_get_time (profile, 50, TRUE);
+	if (value == 6120) {
+		gpm_st_success (test, "got correct time %i", value);
+	} else {
+		gpm_st_failed (test, "got incorrect time %i", value);
+	}
+
+	/************************************************************
+	 **                  NO PROFILE TEST                       **
+	 ************************************************************/
+
+	/* clear old profile */
+	reset_profile (profile);
+
+	/************************************************************/
+	gpm_st_title (test, "single point of accuracy with no guessing (new profile)");
+	gpm_profile_use_guessing (profile, FALSE);
+	gpm_profile_save_percentage (profile, 45, 120, 100);
+	gpm_st_success (test, "put single point");
+
+	/************************************************************/
+	gpm_st_title (test, "make sure we get a correct accuracy when a single point dataset");
+	value = gpm_profile_get_accuracy (profile, 45);
+	if (value == 20) {
+		gpm_st_success (test, "got correct average %i", value);
+	} else {
+		gpm_st_failed (test, "got incorrect average %i", value);
+	}
+
+	/************************************************************/
+	gpm_st_title (test, "make sure we get a correct time when set");
+	value = gpm_profile_get_time (profile, 50, TRUE);
+	if (value == 120) {
+		gpm_st_success (test, "got correct time %i", value);
+	} else {
+		gpm_st_failed (test, "got incorrect time %i", value);
+	}
+
+	/************************************************************/
+	gpm_st_title (test, "make sure we get a non-zero average accuracy when some data present");
+	fvalue = gpm_profile_get_accuracy_average (profile, TRUE);
+	if (fvalue < 0.21 && fvalue > 0.19) {
+		gpm_st_success (test, "got non zero %f", fvalue);
+	} else {
+		gpm_st_failed (test, "got %f", fvalue);
+	}
+
+	/************************************************************
+	 **                  NO PROFILE TEST 2                     **
+	 ************************************************************/
+
+	/* clear old profile */
+	reset_profile (profile);
+
+	/************************************************************/
+	gpm_st_title (test, "single point of accuracy with guessing (new profile)");
+	gpm_profile_use_guessing (profile, TRUE);
+	gpm_profile_save_percentage (profile, 45, 120, 100);
+	gpm_st_success (test, "put single point");
+
+	/************************************************************/
+	gpm_st_title (test, "make sure we get a correct accuracy when a single point dataset");
+	value = gpm_profile_get_accuracy (profile, 45);
+	if (value == 20) {
+		gpm_st_success (test, "got correct average %i", value);
+	} else {
+		gpm_st_failed (test, "got incorrect average %i", value);
+	}
+
+	/************************************************************/
+	gpm_st_title (test, "make sure the point after the guessed point isn't guessed");
+	value = gpm_profile_get_accuracy (profile, 46);
+	if (value == 0) {
+		gpm_st_success (test, "didn't try to guess point after set", value);
+	} else {
+		gpm_st_failed (test, "got accuracy average %i", value);
+	}
+
+	/************************************************************/
+	gpm_st_title (test, "make sure we get a correct time when set");
+	value = gpm_profile_get_time (profile, 50, TRUE);
+	if (value == 5280) {
+		gpm_st_success (test, "got correct time %i", value);
+	} else {
+		gpm_st_failed (test, "got incorrect time!!! %i", value);
+	}
+
+	/************************************************************/
+	gpm_profile_delete_data (profile, FALSE);
+	gpm_profile_delete_data (profile, TRUE);
+
+	g_object_unref (profile);
+
+	gpm_st_end (test);
+}
+
+#endif
+
